unsigned char katakana[]={"ァアィイゥウェエォオカガキギクグケゲコゴサザシジスズセゼソ\ゾタダチヂッツヅテデトドナニヌネノハバパヒビピフブプヘベペホボポマミムメモャヤュユョヨラリルレロヮワヰヱヲンヴヵヶ"};
unsigned char hiragana[]={"ぁあぃいぅうぇえぉおかがきぎくぐけげこごさざしじすずせぜそぞただちぢっつづてでとどなにぬねのはばぱひびぴふぶぷへべぺほぼぽまみむめもゃやゅゆょよらりるれろゎわゐゑをん"};
unsigned char kanatable[]={"ああいいううええおおかかききくくけけここささししすすせせそそたたちちつつつててととなにぬねのはははひひひふふふへへへほほほまみむめもややゆゆよよらりるれろわわゐゑをんうかか"};
unsigned char symboltable[]={"　！”＃＄％＆’（）＊＋，−．／０１２３４５６７８９：；＜＝＞？＠ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ［￥］＾＿‘ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ｛｜｝〜"};
unsigned char btable[6][21]={"あかさたなはまやらわ","いきしちにひみりゐ","うくすつぬふむゆる","えけせてねへめれゑ","おそことのほもよろを","ん"};
