#define AKASATANA "あかさたなはまやらわ"
#define akasatana "アカサタナハマヤラワ"

#define AIUEO "あいうえおかきくけこさしすせそたちつてとなにぬねのはひふへほまみむめもやゆよらりるれろわゐゑをん"
#define aiueo "アイウエオカキクケコサシスセソ\タチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヰヱヲン"

#define ONBIKI "ー"
#define ZERO "０"
#define ALPHAEND "ｚ"
#define HIRATOP "ぁ"
#define HIRAEND "ん"
#define KATATOP "ァ"
#define KATAEND "ヶ"
